#include <cstdlib>
#include <cmath>
#include <iostream>
#include <random>
#include <ctime>
#include "Individual.h"

using namespace std;

// default constructor
// genotype size is a random int from 1 to defaultMaxSize
Individual::Individual()
{
	default_random_engine rng(time(NULL));
	uniform_int_distribution<int> myDist(1,defaultMaxSize);
	int size = myDist(rng);
	
	for(int i = 0; i < size; i++)
		genotype.push_back(randomDouble());

	fitness();
}

// constructor with specified genotype size
Individual::Individual(int msize)
{
	int size = msize;

	for(int i = 0; i < size; i++)
		genotype.push_back(randomDouble());

	fitness();
}

// specified genotype, could be useful for mutation implementations or crossover
Individual::Individual(vector<double> _genotype)
{
	genotype = _genotype;
	fitness();
}

// destructor, empty now; more useful in array-based implementation
Individual::~Individual()
{
}

// return a random double between two specified values
// default values are min/max Ackley function values
double Individual::randomDouble(double _minValue, double _maxValue)
{
	double minValue = _minValue;
	double maxValue = _maxValue;

	double mrandom = minValue + (double)rand()/((double)RAND_MAX/(maxValue - minValue));

	return mrandom;
}

// evaluate fitness using Ackley function
void Individual::fitness()
{
	int size = getSize();

	double firstTerm = 0.0, secondTerm = 0.0;

	for(int i = 1; i <= size; i++)
	{
		firstTerm += genotype[i-1]*genotype[i-1];
		secondTerm += cos(2*PI*genotype[i-1]);
	}

	firstTerm = firstTerm / size;
	secondTerm = secondTerm / size;
	firstTerm = -0.2*sqrt(firstTerm);
	firstTerm = -20*exp(firstTerm);
	secondTerm = exp(secondTerm);

	double funcFitness = firstTerm - secondTerm + 20 + E;

	_fitness = funcFitness;
}

// return fitness
double Individual::getFitness()
{
	return _fitness;
}

// return size
int Individual::getSize()
{
	return genotype.size();
}

// prints the genotype to cout
void Individual::printInd()
{
	cout << endl;
	for(int i = 0; i < getSize(); i++)
		cout << genotype[i] << " ";	
}